@@ -111,24 +111,27 @@ static void   gnome_canvas_line_render      (GnomeCanvasItem *item, GnomeCanvasB
 static GnomeCanvasItemClass *parent_class;
 
 
-GtkType
+GType
 gnome_canvas_line_get_type (void)
 {
-	static GtkType line_type = 0;
+	static GType line_type;
 
 	if (!line_type) {
-		GtkTypeInfo line_info = {
-			"GnomeCanvasLine",
-			sizeof (GnomeCanvasLine),
+		static const GTypeInfo object_info = {
 			sizeof (GnomeCanvasLineClass),
-			(GtkClassInitFunc) gnome_canvas_line_class_init,
-			(GtkObjectInitFunc) gnome_canvas_line_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) gnome_canvas_line_class_init,
+			(GClassFinalizeFunc) NULL,
+			NULL,			/* class_data */
+			sizeof (GnomeCanvasLine),
+			0,			/* n_preallocs */
+			(GInstanceInitFunc) gnome_canvas_line_init,
+			NULL			/* value_table */
 		};
 
-		line_type = gtk_type_unique (gnome_canvas_item_get_type (), &line_info);
+		line_type = g_type_register_static (GNOME_TYPE_CANVAS_ITEM, "GnomeCanvasLine",
+						    &object_info, 0);
 	}
 
 	return line_type;
@@ -145,7 +148,7 @@ gnome_canvas_line_class_init (GnomeCanvasLineClass *class)
 	object_class = (GtkObjectClass *) class;
 	item_class = (GnomeCanvasItemClass *) class;
 
-	parent_class = gtk_type_class (gnome_canvas_item_get_type ());
+	parent_class = g_type_class_peek_parent (class);
 
 	gobject_class->set_property = gnome_canvas_line_set_property;
 	gobject_class->get_property = gnome_canvas_line_get_property;
@@ -336,6 +339,11 @@ get_bounds (GnomeCanvasLine *line, double *bx1, double *by1, double *bx2, double
 	double width;
 	int i;
 
+	if (!line->coords) {
+	    *bx1 = *by1 = *bx2 = *by2 = 0.0;
+	    return;
+	}
+	
 	/* Find bounding box of line's points */
 
 	x1 = x2 = line->coords[0];
@@ -398,15 +406,11 @@ get_bounds (GnomeCanvasLine *line, double *bx1, double *by1, double *bx2, double
 static void
 get_bounds_canvas (GnomeCanvasLine *line, double *bx1, double *by1, double *bx2, double *by2, double affine[6])
 {
-	GnomeCanvasItem *item;
-
 	/* It would be possible to tighten the bounds somewhat by transforming the individual points before
 	   aggregating them into the bbox. But it hardly seems worth it. */
 	ArtDRect bbox_world;
 	ArtDRect bbox_canvas;
 
-	item = GNOME_CANVAS_ITEM (line);
-
 	get_bounds (line, &bbox_world.x0, &bbox_world.y0, &bbox_world.x1, &bbox_world.y1);
 
 	art_drect_affine_transform (&bbox_canvas, &bbox_world, affine);
@@ -867,7 +871,6 @@ gnome_canvas_line_get_property (GObject              *object,
 				GParamSpec           *pspec)
 {
 	GnomeCanvasLine *line;
-	GdkColor *color;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS_LINE (object));
@@ -879,14 +882,21 @@ gnome_canvas_line_get_property (GObject              *object,
 		g_value_set_boxed (value, get_points (line));
 		break;
 
-	case PROP_FILL_COLOR_GDK: {
-		GdkColormap *colormap;
+	case PROP_FILL_COLOR:
+		g_value_set_string_take_ownership (value,
+						   g_strdup_printf ("#%02x%02x%02x",
+								    line->fill_rgba >> 24,
+								    (line->fill_rgba >> 16) & 0xff,
+								    (line->fill_rgba >> 8) & 0xff));
+		break;
 
-		color = g_new (GdkColor, 1);
-		color->pixel = line->fill_pixel;
-		colormap = gtk_widget_get_colormap (GTK_WIDGET (line->item.canvas));
-		gdk_rgb_find_color (colormap, color);
-		g_value_set_boxed (value, color);
+	case PROP_FILL_COLOR_GDK: {
+		GnomeCanvas *canvas = GNOME_CANVAS_ITEM (line)->canvas;
+		GdkColormap *colormap = gtk_widget_get_colormap (GTK_WIDGET (canvas));
+		GdkColor color;
+
+		gdk_colormap_query_color (colormap, line->fill_pixel, &color);
+		g_value_set_boxed (value, &color);
 		break;
 	}
 
@@ -895,9 +905,17 @@ gnome_canvas_line_get_property (GObject              *object,
 		break;
 
 	case PROP_FILL_STIPPLE:
-		g_value_set_boxed (value, line->stipple);
+		g_value_set_object (value, line->stipple);
 		break;
 
+	case PROP_WIDTH_PIXELS:
+		g_value_set_uint (value, line->width);
+		break;
+		
+	case PROP_WIDTH_UNITS:
+		g_value_set_double (value, line->width);
+		break;
+		
 	case PROP_CAP_STYLE:
 		g_value_set_enum (value, line->cap);
 		break;
@@ -1050,19 +1068,19 @@ gnome_canvas_line_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_pa
 
 		gnome_canvas_item_update_svp_clip (item, &line->fill_svp, svp, clip_path);
 
-		if (line->first_arrow)
+		if (line->first_arrow && line->first_coords) {
 			svp = svp_from_points (line->first_coords, NUM_ARROW_POINTS, affine);
-		else
-			svp = NULL;
+                        gnome_canvas_item_update_svp_clip (item, 
+                                        &line->first_svp, svp, clip_path);
+                }
 
-		gnome_canvas_item_update_svp_clip (item, &line->first_svp, svp, clip_path);
 
-		if (line->last_arrow)
+		if (line->last_arrow && line->last_coords) {
 			svp = svp_from_points (line->last_coords, NUM_ARROW_POINTS, affine);
-		else
-			svp = NULL;
+                        gnome_canvas_item_update_svp_clip (item, 
+                                        &line->last_svp, svp, clip_path);
+                }
 
-		gnome_canvas_item_update_svp_clip (item, &line->last_svp, svp, clip_path);
 
 	} else {
 		set_line_gc_foreground (line);
@@ -1107,7 +1125,7 @@ gnome_canvas_line_unrealize (GnomeCanvasItem *item)
 
 static void
 item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coords, int num_points,
-		int *num_drawn_points, double i2c[6])
+		int *num_drawn_points, double i2c[6], int x, int y)
 {
 	int i;
 	int old_cx, old_cy;
@@ -1129,8 +1147,8 @@ item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coord
 	art_affine_point (&pc, &pi, i2c);
 	cx = floor (pc.x + 0.5);
 	cy = floor (pc.y + 0.5);
-	canvas_coords->x = cx;
-	canvas_coords->y = cy;
+	canvas_coords->x = cx - x;
+	canvas_coords->y = cy - y;
 	canvas_coords++;
 	old_cx = cx;
 	old_cy = cy;
@@ -1143,8 +1161,8 @@ item_to_canvas (GnomeCanvas *canvas, double *item_coords, GdkPoint *canvas_coord
 		cx = floor (pc.x + 0.5);
 		cy = floor (pc.y + 0.5);
 		if (old_cx != cx || old_cy != cy) {
-			canvas_coords->x = cx;
-			canvas_coords->y = cy;
+			canvas_coords->x = cx - x;
+			canvas_coords->y = cy - y;
 			old_cx = cx;
 			old_cy = cy;
 			canvas_coords++;
@@ -1175,13 +1193,11 @@ gnome_canvas_line_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	else
 		points = g_new (GdkPoint, line->num_points);
 
+
 	gnome_canvas_item_i2c_affine (item, i2c);
 
-	i2c[4] -= x;
-	i2c[5] -= y;
-
 	item_to_canvas (item->canvas, line->coords, points, line->num_points,
-			&actual_num_points_drawn, i2c);
+			&actual_num_points_drawn, i2c, x, y);
 
 	if (line->stipple)
 		gnome_canvas_set_stipple_origin (item->canvas, line->gc);
@@ -1197,13 +1213,13 @@ gnome_canvas_line_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 
 	if (line->first_arrow) {
 		item_to_canvas (item->canvas, line->first_coords, points, NUM_ARROW_POINTS,
-				&actual_num_points_drawn, i2c);
+				&actual_num_points_drawn, i2c, x, y);
 		gdk_draw_polygon (drawable, line->gc, TRUE, points, actual_num_points_drawn );
 	}
 
 	if (line->last_arrow) {
 		item_to_canvas (item->canvas, line->last_coords, points, NUM_ARROW_POINTS,
-				&actual_num_points_drawn, i2c);
+				&actual_num_points_drawn, i2c, x, y);
 		gdk_draw_polygon (drawable, line->gc, TRUE, points, actual_num_points_drawn );
 	}
 }
