@@ -53,7 +53,7 @@ enum {
 	PROP_X1,
 	PROP_Y1,
 	PROP_X2,
-	PROP_Y2,
+	PROP_Y2
 };
 
 
@@ -69,32 +69,33 @@ static void gnome_canvas_re_get_property (GObject              *object,
 					  GValue               *value,
 					  GParamSpec           *pspec);
 
-static void gnome_canvas_re_update_shared      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
-
 static void gnome_canvas_rect_update      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 static void gnome_canvas_ellipse_update      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 
 static GnomeCanvasItemClass *re_parent_class;
 
 
-GtkType
+GType
 gnome_canvas_re_get_type (void)
 {
-	static GtkType re_type = 0;
+	static GType re_type;
 
 	if (!re_type) {
-		GtkTypeInfo re_info = {
-			"GnomeCanvasRE",
-			sizeof (GnomeCanvasRE),
+		static const GTypeInfo object_info = {
 			sizeof (GnomeCanvasREClass),
-			(GtkClassInitFunc) gnome_canvas_re_class_init,
-			(GtkObjectInitFunc) gnome_canvas_re_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) gnome_canvas_re_class_init,
+			(GClassFinalizeFunc) NULL,
+			NULL,			/* class_data */
+			sizeof (GnomeCanvasRE),
+			0,			/* n_preallocs */
+			(GInstanceInitFunc) gnome_canvas_re_init,
+			NULL			/* value_table */
 		};
 
-		re_type = gtk_type_unique (gnome_canvas_shape_get_type (), &re_info);
+		re_type = g_type_register_static (GNOME_TYPE_CANVAS_SHAPE, "GnomeCanvasRE",
+						  &object_info, 0);
 	}
 
 	return re_type;
@@ -105,13 +106,11 @@ gnome_canvas_re_class_init (GnomeCanvasREClass *class)
 {
 	GObjectClass *gobject_class;
 	GtkObjectClass *object_class;
-	GnomeCanvasItemClass *item_class;
 
 	gobject_class = (GObjectClass *) class;
 	object_class = (GtkObjectClass *) class;
-	item_class = (GnomeCanvasItemClass *) class;
 
-	re_parent_class = gtk_type_class (gnome_canvas_shape_get_type ());
+	re_parent_class = g_type_class_peek_parent (class);
 
 	gobject_class->set_property = gnome_canvas_re_set_property;
 	gobject_class->get_property = gnome_canvas_re_get_property;
@@ -157,13 +156,9 @@ gnome_canvas_re_init (GnomeCanvasRE *re)
 static void
 gnome_canvas_re_destroy (GtkObject *object)
 {
-	GnomeCanvasRE *re;
-
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS_RE (object));
 
-	re = GNOME_CANVAS_RE (object);
-
 	if (GTK_OBJECT_CLASS (re_parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (re_parent_class)->destroy) (object);
 }
@@ -250,41 +245,32 @@ gnome_canvas_re_get_property (GObject              *object,
 	}
 }
 
-static void
-gnome_canvas_re_update_shared (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
-{
-	GnomeCanvasRE *re;
-
-	re = GNOME_CANVAS_RE (item);
-
-	if (re_parent_class->update)
-		(* re_parent_class->update) (item, affine, clip_path, flags);
-}
-
-
 /* Rectangle item */
 static void gnome_canvas_rect_class_init (GnomeCanvasRectClass *class);
 
 
 
-GtkType
+GType
 gnome_canvas_rect_get_type (void)
 {
-	static GtkType rect_type = 0;
+	static GType rect_type;
 
 	if (!rect_type) {
-		GtkTypeInfo rect_info = {
-			"GnomeCanvasRect",
-			sizeof (GnomeCanvasRect),
+		static const GTypeInfo object_info = {
 			sizeof (GnomeCanvasRectClass),
-			(GtkClassInitFunc) gnome_canvas_rect_class_init,
-			(GtkObjectInitFunc) NULL,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) gnome_canvas_rect_class_init,
+			(GClassFinalizeFunc) NULL,
+			NULL,			/* class_data */
+			sizeof (GnomeCanvasRect),
+			0,			/* n_preallocs */
+			(GInstanceInitFunc) NULL,
+			NULL			/* value_table */
 		};
 
-		rect_type = gtk_type_unique (gnome_canvas_re_get_type (), &rect_info);
+		rect_type = g_type_register_static (GNOME_TYPE_CANVAS_RE, "GnomeCanvasRect",
+						    &object_info, 0);
 	}
 
 	return rect_type;
@@ -308,8 +294,8 @@ gnome_canvas_rect_update (GnomeCanvasItem *item, double affine[6], ArtSVP *clip_
 
 	re = GNOME_CANVAS_RE(item);
 
-	if(re->path_dirty) {		
-		path_def = gnome_canvas_path_def_new();
+	if (re->path_dirty) {		
+		path_def = gnome_canvas_path_def_new ();
 		
 		gnome_canvas_path_def_moveto(path_def, re->x1, re->y1);
 		gnome_canvas_path_def_lineto(path_def, re->x2, re->y1);
@@ -317,13 +303,13 @@ gnome_canvas_rect_update (GnomeCanvasItem *item, double affine[6], ArtSVP *clip_
 		gnome_canvas_path_def_lineto(path_def, re->x1, re->y2);
 		gnome_canvas_path_def_lineto(path_def, re->x1, re->y1);		
 		gnome_canvas_path_def_closepath_current(path_def);		
-		gnome_canvas_shape_set_path_def(G_OBJECT(item), path_def);
+		gnome_canvas_shape_set_path_def (GNOME_CANVAS_SHAPE (item), path_def);
 		gnome_canvas_path_def_unref(path_def);
 		re->path_dirty = 0;
 	}
 
-	gnome_canvas_re_update_shared (item, affine, clip_path, flags);
-
+	if (re_parent_class->update)
+		(* re_parent_class->update) (item, affine, clip_path, flags);
 }
 
 /* Ellipse item */
@@ -332,24 +318,27 @@ gnome_canvas_rect_update (GnomeCanvasItem *item, double affine[6], ArtSVP *clip_
 static void gnome_canvas_ellipse_class_init (GnomeCanvasEllipseClass *class);
 
 
-GtkType
+GType
 gnome_canvas_ellipse_get_type (void)
 {
-	static GtkType ellipse_type = 0;
+	static GType ellipse_type;
 
 	if (!ellipse_type) {
-		GtkTypeInfo ellipse_info = {
-			"GnomeCanvasEllipse",
-			sizeof (GnomeCanvasEllipse),
+		static const GTypeInfo object_info = {
 			sizeof (GnomeCanvasEllipseClass),
-			(GtkClassInitFunc) gnome_canvas_ellipse_class_init,
-			(GtkObjectInitFunc) NULL,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) gnome_canvas_ellipse_class_init,
+			(GClassFinalizeFunc) NULL,
+			NULL,			/* class_data */
+			sizeof (GnomeCanvasEllipse),
+			0,			/* n_preallocs */
+			(GInstanceInitFunc) NULL,
+			NULL			/* value_table */
 		};
 
-		ellipse_type = gtk_type_unique (gnome_canvas_re_get_type (), &ellipse_info);
+		ellipse_type = g_type_register_static (GNOME_TYPE_CANVAS_RE, "GnomeCanvasEllipse",
+						       &object_info, 0);
 	}
 
 	return ellipse_type;
@@ -371,67 +360,72 @@ static void
 gnome_canvas_ellipse_update (GnomeCanvasItem *item, double affine[6], ArtSVP *clip_path, gint flags) {
 	GnomeCanvasPathDef *path_def;
 	GnomeCanvasRE *re;
-#if 0
-	int i = 0;
-	double th;
-#endif
 
 	re = GNOME_CANVAS_RE(item);
 
-	if(re->path_dirty) {
-#if 1
+	if (re->path_dirty) {
 		gdouble cx, cy, rx, ry;
-#endif
+		gdouble beta = 0.26521648983954400922; /* 4*(1-cos(pi/8))/(3*sin(pi/8)) */
+		gdouble sincosA = 0.70710678118654752440; /* sin (pi/4), cos (pi/4) */
+		gdouble dx1, dy1, dx2, dy2;
+		gdouble mx, my;
 
 		path_def = gnome_canvas_path_def_new();
-#if 0
-		th = (2 * M_PI * i) / N_PTS;
-		
-		gnome_canvas_path_def_moveto(path_def, (re->x1 + re->x2) * 0.5 + (re->x2 - re->x1) * 0.5 * cos (th), (re->y1 + re->y2) * 0.5 - (re->y2 - re->y1) * 0.5 * sin (th));       
-		
-		for (i = 1; i < N_PTS + 1; i++) {
-			th = (2 * M_PI * i) / N_PTS;		
-			gnome_canvas_path_def_lineto(path_def,
-						     (re->x1+ re->x2) * 0.5 + (re->x2 - re->x1) * 0.5 * cos (th), 
-						     (re->y1 + re->y2) * 0.5 - (re->y2 - re->y1) * 0.5 * sin (th));	
-		}
-#else
-
-#define EBD 0.552
 
 		cx = (re->x2 + re->x1) * 0.5;
 		cy = (re->y2 + re->y1) * 0.5;
 		rx = re->x2 - cx;
 		ry = re->y2 - cy;
 
-		gnome_canvas_path_def_moveto (path_def, cx - rx, cy);
+		dx1 = beta * rx;
+		dy1 = beta * ry;
+		dx2 = beta * rx * sincosA;
+		dy2 = beta * ry * sincosA;
+		mx = rx * sincosA;
+		my = ry * sincosA;
+
+		gnome_canvas_path_def_moveto (path_def, cx + rx, cy);
 		gnome_canvas_path_def_curveto (path_def,
-					       cx - rx, cy + ry * EBD,
-					       cx - rx * EBD, cy + ry,
-					       cx, cy + ry);
+					       cx + rx, cy - dy1,
+					       cx + mx + dx2, cy - my + dy2,
+					       cx + mx, cy - my);
 		gnome_canvas_path_def_curveto (path_def,
-					       cx + rx * EBD, cy + ry,
-					       cx + rx, cy + ry * EBD,
-					       cx + rx, cy);
-		gnome_canvas_path_def_curveto (path_def,
-					       cx + rx, cy - ry * EBD,
-					       cx + rx * EBD, cy - ry,
+					       cx + mx - dx2, cy - my - dy2,
+					       cx + dx1, cy - ry,
 					       cx, cy - ry);
 		gnome_canvas_path_def_curveto (path_def,
-					       cx - rx * EBD, cy - ry,
-					       cx - rx, cy - ry * EBD,
+					       cx - dx1, cy - ry,
+					       cx - mx + dx2, cy - my - dy2,
+					       cx - mx, cy - my);
+		gnome_canvas_path_def_curveto (path_def,
+					       cx - mx - dx2, cy - my + dy2,
+					       cx - rx, cy - dy1,
 					       cx - rx, cy);
-#endif
-
-#undef EBD
+		
+		gnome_canvas_path_def_curveto (path_def,
+					       cx - rx, cy + dy1,
+					       cx - mx - dx2, cy + my - dy2,
+					       cx - mx, cy + my);
+		gnome_canvas_path_def_curveto (path_def,
+					       cx - mx + dx2, cy + my + dy2,
+					       cx - dx1, cy + ry,
+					       cx, cy + ry);
+		gnome_canvas_path_def_curveto (path_def,
+					       cx + dx1, cy + ry,
+					       cx + mx - dx2, cy + my + dy2,
+					       cx + mx, cy + my);
+		gnome_canvas_path_def_curveto (path_def,
+					       cx + mx + dx2, cy + my - dy2,
+					       cx + rx, cy + dy1,
+					       cx + rx, cy);
 		
 		gnome_canvas_path_def_closepath_current(path_def);
 		
-		gnome_canvas_shape_set_path_def(G_OBJECT(item), path_def);
+		gnome_canvas_shape_set_path_def (GNOME_CANVAS_SHAPE (item), path_def);
 		gnome_canvas_path_def_unref(path_def);
 		re->path_dirty = 0;
 	}
 
-		
-	gnome_canvas_re_update_shared(item, affine, clip_path, flags);
+	if (re_parent_class->update)
+		(* re_parent_class->update) (item, affine, clip_path, flags);
 }
